#!/bin/bash
#
# This script executes a prog under valgrind
#
# $Id: valgrind.sh,v 1.11 2009/07/01 06:41:19 zlb Exp $

PHG_opts="-mem_sampling_rate=0 -no_perfctr"

if test $# -lt 2; then
    echo "Usage: $0 nprocs prog arguments"
    exit 1
fi

. `dirname $0`/functions

prog=`canonicalize $2`
nprocs=$1
shift 2

supp=`dirname $0`/valgrind.supp
suppressions="--gen-suppressions=no --suppressions=`canonicalize $supp`"
curdir=`pwd`
out="valgrind-`hostname -s`"

cat <<END >valgrind_wrapper.sh
#!/bin/bash
#
export GLIBCPP_FORCE_NEW=1	# gcc >= 3.2.2
export GLIBCXX_FORCE_NEW=1	# gcc >= 3.4
yes 2>/dev/null | valgrind --tool=memcheck --log-file=${curdir}/$out \
	--leak-check=yes --show-reachable=yes --run-libc-freeres=yes \
	$suppressions $prog "\$@"
exit \$?
END
chmod a+x valgrind_wrapper.sh

if false; then
    # for MPICH-1 using P4 group file
    echo "localhost 0 `pwd`/valgrind_wrapper.sh" >p4file
    i=1
    while [ $i -lt $nprocs ]; do
	echo "localhost 1 `pwd`/valgrind_wrapper.sh" >>p4file
	i=$((i+1))
    done
    trap "/bin/rm -f p4file valgrind_wrapper.sh" 0 1 2 3 15
    /bin/rm -f valgrind[0-9]*.*[0-9]*
    if test $nprocs -eq 1; then
	bash valgrind_wrapper.sh $PHG_opts "$@"
    else
	bash valgrind_wrapper.sh -p4pg p4file $PHG_opts "$@"
    fi
else
    # use mpirun
    trap "/bin/rm -f valgrind_wrapper.sh" 0 1 2 3 15
    /bin/rm -f ${out}.*[0-9]*
    if which mpirun >/dev/null 2>&1; then
	mpirun -np $nprocs ./valgrind_wrapper.sh $PHG_opts "$@"
    elif which mpiexec >/dev/null 2>&1; then
	mpiexec -n $nprocs ./valgrind_wrapper.sh $PHG_opts "$@"
    else
	bash valgrind_wrapper.sh $PHG_opts "$@"
    fi
fi
